#ifndef LO_SIMPLEX
#define LO_SIMPLEXC
#include "lo_names.h"
#include LO_SIMPLEX_H

lo_Simplex::lo_Simplex(char *nam):eps(ls_EPS),it(0)
{ strcpy(name=ONAME,nam);
  A_name=A.name; T_name=T.name; c_name=c.name; b_name=b.name;
  basis_name=b.name; basis_name=basis.name; nbasis_name=nbasis.name;
  strcpy(A_name,Constraints_Matrix_);
  strcpy(T_name,Simplex_Table_);
  strcpy(c_name,Objective_Function_);
  strcpy(b_name,Right_Hand_Side_);
  strcpy(basis_name,Basis_Indices_);
  strcpy(nbasis_name,Nonbasis_Indices_);

}

lo_Simplex::lo_Simplex(lo_Simplex &B)
{ eps=B.eps; strcpy(name=ONAME,B.name);
  A_name=A.name; T_name=T.name; c_name=c.name; b_name=b.name;
  basis_name=b.name; basis_name=basis.name; nbasis_name=nbasis.name;
}

/*
  Aufbau des Rechenschemas fuer die Simplexmethode;
  es hat n+1 Spalten und m+1 Zeilen.
*/
lo_Simplex::lo_Simplex(ls_Vector &cc,ls_Matrix &AA,ls_Vector &bb,char *nam):
eps(ls_EPS)
{ static char *q1="lo_Simplex::lo_Simplex";
  strcpy(name=ONAME,nam);
  A_name=A.name; T_name=T.name; c_name=c.name; b_name=b.name;
  basis_name=b.name; basis_name=basis.name; nbasis_name=nbasis.name;
  int m=(int)AA.number_of_rows(), n=(int)AA.number_of_columns(),i, j;
  if(m!=(int)bb.dimension()) throw ls_error(bb.name,q1,size_error_);
  if(n!=(int)cc.dimension()) throw ls_error(cc.name,q1,size_error_);
  A.swap(AA); strcpy(A.name,Constraints_Matrix_);
  c.swap(cc); strcpy(c.name,Objective_Function_);
  b.swap(bb); strcpy(b.name,Right_Hand_Side_);
/*
  Die Indices der natuerlichen Variablen sind 1,...,n
*/
  ls_array1<int> nb(n,Nonbasis_Indices_);
  for(j=0; j<n; nb[j]=j+1, j++);
  nb.swap(nbasis);
/*
  Die Indices der Schlupfvariablen sind -1,-2,...,-m
*/
  ls_array1<int> ba(m,Basis_Indices_);
  for(i=0; i<m; ba[i]=-(i+1), i++);
  ba.swap(basis);

/*
  Koeffizietenmatrix in T ab Position (0,0)
*/
  ls_Matrix TT(m+1,n+1,Simplex_Table_); TT.put_array(A);
/*
  Fuer die Zielfunktionskoeffizienten ist die letzte Zeile reserviert;
  die rechte Seite kommt in die letzte Spalte (ab Zeile 0;
  ab 1. Zeile, 1. Spalte steht die Koeffizientenmatrix.
*/
  TT.put_column(b,0,n); T.swap(TT); it=0;
}

/*
  Loesen eines LO-Problems in Normalform, d. h. alle Variablen duerfen
  nur nichtnegative Werte haben, die Restriktionen sind saemtlich in
  Ungleichungsform und die Zielfunktion ist zu maximieren.
*/
ls_Vector lo_Simplex::max_with_inequalities(ls_REAL &zfw, ls_Vector &x)
{ static char *q1="lo_Simplex::max_with_inequalities";
  int n=(int)T.number_of_columns(), m=(int)T.number_of_rows(),
      i, j,
      is,       // Index einer Pivotzeile
      js,       // Index einer Pivotspalte
      iz=m-1,   // Zeilen-Index fuer Zielfunktion
      jh=0,     // Spalten-Index fuer Hilfsspalte
      jb=n-1,   // Spalten-Index der rechten Seite
      ires=m-1; // Anzahl der Restriktionen
  if(!m||!n) throw ls_error(name,q1,size_error_);
  ls_REAL min=-eps, s, EPS=(ls_REAL)(1./eps);
/*
   Untersuchung, ob eine rechte Seite negativ ist;
   gegebenenfalls werden eine Hilfszeile und
   eine Hilfsspalte eingefuegt (Einfuehren einer Hilfsvariablen);
   die Hilfsvariable erhaelt den Index n.
*/
  if(it>0) throw ls_error(name,q1,problem_is_solved_);
  if(!it) T.put_row(-c,iz); else T.put_row(c,iz);
  for(is=i=0; i<ires; i++)
    if((min=T[i][jb])<0)
    { T.append_array(); T.swap_column(n-1,n);
      nbasis.append(); nbasis[n-1]=n; m++, n++;
      jh=n-2; iz++; jb++; T[i][jh]=-1;
      if(!it) T[iz][jh]=1; else T[iz][jh]=-1;
      for(is=i, j=i+1; j<ires; j++)
        if((s=T[j][jb])<0)
        { T[j][jh]=-1; if(s<min) is=j, min=s;}
      break;
    }
/*
  Falls eine Hilfsvariable benutzt wird, ist eine erste
  Transformation auszufuehren, die das Schema primal zulaessig
  macht.
*/
  if(jh)
  { for(js=jh, j=0; j<n; T[is][j]=-T[is][j], j++);
    for(T[is][js]=-1, i=0; i<m; i++)
      if((i!=is)&&T[i][js])
      { for(T[i][js]=0, j=0; j<n; T[i][j]-=T[is][j], j++);
        T[i][js]=1;
      }
    j=basis[is], basis[is]=nbasis[js], nbasis[js]=j;
  }
  it=0;
/*
  Simplexmethode
*/
  while(min<0)
  {
/*
  Auswahl der Pivotspalte
*/
    for(min=-eps, js=-1, j=0; j<jb; j++)
      if((s=T[iz][j])<min) min=s, js=j;
    if(js<0)
    { if(jh) throw ls_error(name,q1,no_feasible_solution_);
      break;
    }
/*
  Auswahl der Pivotzeile
*/
    for(min=EPS, is=-1, i=0; i<ires; i++)
      if(((s=T[i][js])>eps)&&((s=(ls_REAL)(T[i][jb]/s))<min)) min=s, is=i;
    if(is<0) throw ls_error(name,q1,objective_is_unbounded_);
/*
  Transformation des Schemas
*/
    s=(ls_REAL)(1./T[is][js]);
    for(T[is][js]=1, j=0; j<n; j++) T[is][j]=(ls_REAL)(s*T[is][j]);
    for(i=0; i<m; i++)
      if((i!=is)&&(fabs(s=-T[i][js])>eps))
        for(T[i][js]=0, j=0; j<n; T[i][j]+=(ls_REAL)(s*T[is][j]), j++);
    j=basis[is], basis[is]=nbasis[js], nbasis[js]=j;
/*
  Falls eine evt. vorhandene Hilfsvariable aus der Basis faellt,
  werden zusaetzliche Zeile und Spalte entfernt, da das
  so reduzierte Schema primal zulaessig fuer die urspruengliche
  Aufgabe ist.
*/
    if(jh&&(j==n-1))
    { T.remove_column(js); T.remove_row(iz); nbasis.remove(js);
      iz--; m--; jh=0; jb--; n--;
    }
/*
*/
    min=-eps; it++;
  }
/*
  Falls die evt. eingefuehrte Hilfsvariable noch in der Basis steht,
  hat die urspruengliche Aufgabe keine zulaessige Loesung.
*/
  if(jh) throw ls_error(name,q1,no_feasible_solution_);
/*
  strcpy(T_name,Optimal_Simplex_Table_);
  Uebergabe der gefundenen optimalen Loesung
  und des Zielfunktionswertes
*/
  ls_Vector y(ires,Constraints_Values_);
  x.put_array(0.);
  for(i=0; i<ires; i++)
    if((j=basis[i])>0) x[j-1]=T[i][jb]; else y[-j-1]=T[i][jb];
  zfw=T[iz][jb];
  return y;
}

/*
  Loesen eines LO-Problems in Normalform, d. h. alle Variablen duerfen
  nur nichtnegative Werte haben, die Restriktionen sind saemtlich in
  Ungleichungsform und die Zielfunktion ist zu minimieren.
*/
ls_Vector lo_Simplex::min_with_inequalities(ls_REAL &zfw, ls_Vector &x)
{ it=-1; ls_Vector y(max_with_inequalities(zfw,x)); zfw=-zfw; return y;
}

lo_Simplex& lo_Simplex::max_with_equations(ls_REAL &zfw, ls_Vector &x)
/*
  Loesen eines LO-Problems in Standardform (Gleichungsform)
  und Maximierung
*/
{ static char *q1="lo_Simplex::max_with_equations";
  int n=(int)T.number_of_columns(), m=(int)T.number_of_rows(),
      i, j, nn=n-1,
      is,       // Index einer Pivotzeile
      js,       // Index einer Pivotspalte
      iz=m-1,   // Zeilen-Index fuer Zielfunktion
      jh=0,     // Spalten-Index fuer Hilfsspalte
      jb=n-1,   // Spalten-Index der rechten Seite
      ires=m-1; // Anzahl der Restriktionen
  if(!m||!n) throw ls_error(name,q1,size_error_);
  ls_REAL min=-eps, s, EPS=1./eps;
/*
  primal-duale Simplexmethode
*/
  if(it>0) throw ls_error(name,q1,problem_is_solved_);
  if(!it) T.put_row(-c,iz); else T.put_row(c,iz);
  it=0;
  while(min<0)
  {
/*
  Auswahl einer Pivotspalte, die zu einer natuerlichen Variablen gehoert.
*/
    for(js=-1, min=-eps, j=0; j<jb; j++)
      if((nbasis[j]>0)&&((s=T[iz][j])<min)) min=s, js=j;

    if(js<0)
/*
  Es wurde keine Pivotspalte gefunden; nun wird nach einer
  Pivotzeile gesucht, die einer noch nicht aufgeloesten Gleichung
  entspricht, um diese nach einer natuerlichen Variablen aufzuloesen.
*/
    { for(is=-1, min=-eps, i=0; i<ires; i++)
        if((basis[i]<0)&&((s=(ls_REAL)fabs(T[i][jb]))>min)) min=s, is=i;

      if(is<0)
/*
  Alle urspruenglichen Gleichungen sind aufgeloest,
  also ist das Schema optimal, da kein modifizierter
  Zielfunktionskoeffizient negativ ist.
*/
        break;
/*
  Eine Pivotszeile ist gefunden.
  Es wird ein dualer Austauschschritt vorbereitet; dazu
  muss die betreffende rechte Seite der Gleichung negativ sein.
*/
      if(T[is][jb]>0)
        for(j=0; j<jb; j++) T[is][j]=-T[is][j];
/*
  Auswahl einer Pivotspalte
*/
      for(js=-1, min=EPS, j=0; j<jb; j++)
        if((nbasis[j]>0)&&((s=T[is][j])+eps<0)&&
          ((s=(ls_REAL)(T[iz][j]/fabs(s)))<min)) min=s, js=j;
      if(js<0) throw ls_error(name,q1,problem_unsolvable_);
    }
    else
/*
  Ein primaler Austauschschritt liegt an.
  Auswahl einer Pivotzeile; dazu muessten die noch nicht aufgeloesten
  Gleichungen aus der Aufgabe nichtnegative, rechte Seiten haben;
  also muss das Pivotelement bei einer negativen rechten Seite positiv
  und bei einer positiven rechten Seite negativ sein.
*/
    { for(is=-1, min=EPS, i=0; i<ires; i++)
      { if((basis[i]<0)&&(T[i][jb]<0)) s=-T[i][js]; else s=T[i][js];
        if((s>eps)&&((s=(ls_REAL)(fabs(T[i][jb]/s)))<min)) min=s, is=i;
      }
      if(is<0) throw ls_error(name,q1,objective_is_unbounded_);
    }
/*
  Transformation
*/
    s=(ls_REAL)(1./T[is][js]);
    for(T[is][js]=1, j=0; j<n; T[is][j]=(ls_REAL)(s*T[is][j]), j++);
    for(i=0; i<m; i++)
      if((i!=is)&&(fabs(s=-T[i][js])>eps))
        for(T[i][js]=0, j=0; j<n; T[i][j]+=(ls_REAL)(s*T[is][j]), j++);
    j=basis[is], basis[is]=nbasis[js], nbasis[js]=j;
/*
    Falls eine der urspruenglichen Gleichungen aufgeloest wurde,
    wird die entsprechende Spalte gestrichen.
*/
    if(j<0) T.remove_column(js), nbasis.remove(js), n--, jb--;

    min=-eps; it++;
  }
  strcpy(T_name,Optimal_Simplex_Table_);
/*
  Loesungsausgabe
*/
  x.put_array(0.);
  for(i=0; i<ires; i++)
    if((is=basis[i])>0)
    { if(T[i][jb]<0) for(j=0; j<n; T[i][j]=-T[i][j], j++);
      x[is-1]=T[i][jb];
    }

  zfw=T[iz][jb];
  return *this;
}

lo_Simplex& lo_Simplex::min_with_equations(ls_REAL &zfw, ls_Vector &x)
{ it=-1; max_with_equations(zfw,x); zfw=-zfw; return *this;}

const lo_Simplex& lo_Simplex::operator=(const lo_Simplex &B)
{ eps=B.eps; it=B.it; A=B.A; basis=B.basis; nbasis=B.nbasis;
  T=B.T, c=B.c, b=B.b; return *this;
}

const lo_Simplex& lo_Simplex::write(ostream &f) const
{ f<<name<<endl<<"{"<<endl;
  c.write(f); A.write_row(f); b.write(f); T.write_row(f);
  basis.write(f); nbasis.write(f);
  f<<"}"<<endl; return *this;
}

lo_Simplex& lo_Simplex::read(istream &f)
{ static char *q1="lo_Simplex::read";
  char w[128];
  if(*name) while(f>>w&&strcmp(w,name));
  else f>>name, *(c.name)=0, *(A.name)=0, *(b.name)=0,
    *(basis.name)=0, *(nbasis.name)=0;
  if(!f.good()) throw ls_error(name, q1, object_not_found_);
  f>>w;
  if(strcmp(w, "{")) throw ls_error(name,q1,object_contents_error_);
  c.read(f); A.read_row(f); b.read(f); T.read_row(f);
  basis.read(f); nbasis.read(f);
  if(!(f>>w)||(*w!='}')) throw ls_error(name, q1, object_end_error_);
  return *this;
}

const lo_Simplex& lo_Simplex::operator>> (char *nam) const
{ int l;
  switch ((unsigned char)(sizeof(ls_REAL)))
  { case  4: l=5; break;
    case  8: l=12; break;
    case 16: l=28; break;
  }
  if(!*nam)
    cout.setf(ios::scientific, ios::floatfield),
    cout.precision(l), write(cout);
  else
  { ofstream f(nam);
    f.setf(ios::scientific, ios::floatfield);
    f.precision(l); write(f);
  }
  return *this;
}

lo_Simplex& lo_Simplex::operator<< (char *nam)
{ if(!*nam) read(cin); else{ ifstream f(nam); read(f);}return *this;}

#endif
