#ifndef LO_SIMPLEX
#define LO_SIMPLEX
#include "lo_names.h"
#ifndef LS_MATRIX
#include LS_MATRIX_H // LS-System wird bentigt!
#endif

/*
Naive Simplexmethode (Ohne numerische Ansprche!):
Man maximiere/minimiere (c,x) unter Ungleichungsbedingungen Ax<=b, x>=0
oder unter Gleichungsbedingungen Ax=b,x>=0:
*/

class lo_Simplex
{ protected:
  ls_Matrix A, T;                 // Problem-Matrix und Simplex-Tableau
  ls_Vector c, b;                 // ZF- und rechte-Seiten-Vektor
  ls_array1<int> basis, nbasis;   // Indices der Basis-, Nichtbasisvar.
  char ONAME[ls_len];             // Objektname
  public:
  char *name, *A_name, *T_name, *c_name, 
       *b_name, *basis_name, *nbasis_name;
                                  // Zeiger auf Objektnamen
  ls_REAL eps;                    // Genauigkeitsschranke
  int it;                         // Anzahl der ausgefhrten Iterationen
  lo_Simplex(char *nam="lo_Simplex");
                                  // Standard-Konstruktor mit Objektnamen
  lo_Simplex(ls_Vector &cc, ls_Matrix &AA, 
             ls_Vector &bb, char* ="lo_Simplex");
                                  // Datenbernahme; danach sind
                                  // die Objekte cc, AA, bb ohne Daten
  lo_Simplex(lo_Simplex &);
  const lo_Simplex& operator=(const lo_Simplex &);
  ls_Vector min_with_inequalities(ls_REAL &zfw, ls_Vector &x);
  ls_Vector max_with_inequalities(ls_REAL &zfw, ls_Vector &x);
                                  // Modell in Ungleichungsform;
                                  // auf x werden die optimale Lsung
                                  // und auf zfw der optimale
                                  // Zielfunktionswert abgeliefert;
                                  // Rckkehrwert ist ein Vektor,
                                  // der die Erfllung der Restriktionen
                                  // zeigt.
  lo_Simplex& min_with_equations(ls_REAL &, ls_Vector &x);
  lo_Simplex& max_with_equations(ls_REAL &, ls_Vector &x);
                                  // Modell in Gleichungsform;
                                  // es wird ein primal-dual-Algorithmus
                                  // angewendet.
  ls_Vector balance(ls_REAL &zfw, const ls_Vector &x) const
  { ls_Vector r; r=b-A*x; zfw=c*x; return r;}
                                  // Zu einem gegebenen Vektor werden die
                                  // Erfllung der Restriktionen und der
                                  // Zielfunktionswert ausgegeben.
  const lo_Simplex& write(ostream &) const;
                                  // Objektdaten-Ausgabe in eine Datei
  lo_Simplex& read(istream &);    // Objektdaten-Eingabe aus einer Datei
  const lo_Simplex& operator>>(char *) const;
                                  // Ausgabe in eine namentliche Datei
  lo_Simplex& operator<<(char *); // Eingabe aus einer namentlichen Datei
};
#ifndef LO_LIB
#ifndef LO_SIMPLEXC
#include LS_SIMPLEX_C
#endif
#endif
#endif
