//: $CC -DLS_LIB -c $file
#ifndef LS_ARRAY2C
#define LS_ARRAY2C
#include "ls_names.h"
#include LS_ARRAY2_H

template<class T>
ls_array2<T>::ls_array2(ls_UINT mm, ls_UINT nn, char *nam):
m(mm),n(nn),ls_array1<T>(mm*nn, nam){}

template<class T>
ls_array2<T>::ls_array2(ls_array2<T> &A):ls_array1<T>(A)
{ m=A.number_of_rows(),n=A.number_of_columns(); A.set_data();}

template<class T>
ls_array2<T>::ls_array2(T *A, ls_UINT mm, ls_UINT nn, char *nam):
m(mm),n(nn),ls_array1<T>(A, mm*nn,nam){}

template<class T>
ls_array2<T>& ls_array2<T>::swap(ls_array2<T> &A)
{ ls_array1<T>::swap(A); ls_UINT mm=m, nn=n;
  m=A.number_of_rows(), n=A.number_of_columns();
  A.set_data(mm,nn); return *this;
}

template<class T>
T* ls_array2<T>::operator[](ls_UINT i) const
{
#ifdef LS_TEST
  static char *q1="ls_array2::[]";
  if(!(i<m)) throw ls_error(name,q1,index_error_);
#endif
  return a+i*n;
}

template<class T>
const ls_array2<T>& ls_array2<T>::operator=(const ls_array2<T> &v)
{ static char *q1="ls_array2::=";
  ls_UINT mm=v.number_of_rows(), nn=v.number_of_columns();
  if(!m) m=mm, n=nn;
  if((m!=mm)||(n!=nn)) throw ls_error(name,q1,size_error_);
  ls_array1<T>::operator=(v);
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put(T v, ls_UINT i, ls_UINT j)
{
#ifdef LS_TEST
  static char *q1="ls_array2::put";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
#endif
  a[i*n+j]=v; return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_row(const ls_array1<T> &v, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_row";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT vdim=v.dimension(), ll=(vdim<n-j)? vdim:n-j;
  memcpy(a+i*n+j, v.asArray(), sizeof(T)*ll);
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_row(T s, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_row";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  if(!s) memset(a+i*n+j,0, sizeof(T)*(n-j));
  else{ T *aa=a+i*n; for(; j<n; aa[j++]=s);}
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_column(const ls_array1<T> &v, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_column";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT vdim=v.dimension(),ll=(m-i<vdim)? m-i:vdim;
  T *aa=a+i*n+j, *vv=v.asArray();
  while(ll--) *aa=*vv, aa+=n, vv++;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_column(T s, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_column";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  T *aa=a+i*n+j, *ae=a+dim;
  while(aa<ae) *aa=s, aa+=n;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_diagonal(const ls_array1<T> &v, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_diagonal";
  if(!(j<n)||!(i<m)) throw ls_error(name,q1,index_error_);
  ls_UINT vdim=v.dimension(), ll;
  if((i+n<=m)||(m-i>n-j)) ll=n-j; else ll=m-i;
  if(vdim<ll) ll=vdim;
  T *aa=a+i*n+j, *vv=v.asArray();
  while(ll--) *aa=*vv, aa+=n+1, vv++;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_diagonal(T s, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_diagonal";
  if(!(j<n)||!(i<m)) throw ls_error(name,q1,size_error_);
  T *aa=a+i*n+j, *ae=a+dim;
  while(aa<ae) *aa=s, aa+=n+1;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_array(const ls_array2<T> &v, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_array";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT mm=m-i, nn=n-j, mv=v.number_of_rows(), nv=v.number_of_columns();
  if(!mv||!nv) throw ls_error(v.name,q1,object_is_empty_);
  if(mm>mv) mm=mv; if(nn>nv) nn=nv;
  nn*=(ls_UINT)sizeof(T);
  T *aa=a+i*n+j, *vv=v.asArray();
  while(mm--) memcpy(aa,vv,nn), aa+=n, vv+=nv;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::put_array(T s, ls_UINT i, ls_UINT j)
{ static char *q1="ls_array2::put_array";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  for(T *aa=a+i*n, *ae=a+dim, *aae; aa<ae;)
     for(aae=aa+n,aa+=j; aa<aae; *aa=s, aa++);
  return *this;
}

template<class T>
T ls_array2<T>::get(ls_UINT i, ls_UINT j) const
{
#ifdef LS_TEST
  static char *q1="ls_array2::get";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
#endif
  return a[i*n+j];
}

template<class T>
const ls_array2<T>& ls_array2<T>::\
get_row(ls_array1<T> &v, ls_UINT i, ls_UINT j) const
{ static char *q1="ls_array2::get_row";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT vdim=v.dimension(), ll=(n-j<vdim)? n-j:vdim;
  memcpy(v.asArray(), a+i*n+j, sizeof(T)*ll);
  return *this;
}

template<class T>
const ls_array2<T>& ls_array2<T>::\
get_column(ls_array1<T> &v, ls_UINT i, ls_UINT j) const
{ static char *q1="ls_array2::get_column";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT vdim=v.dimension(),ll=(m-i<vdim)? m-i:vdim;
  T *aa=a+i*n+j, *vv=v.asArray();
  while(ll--) *vv=*aa, aa+=n, vv++;
  return *this;
}

template<class T>
const ls_array2<T>& ls_array2<T>::\
get_diagonal(ls_array1<T> &v, ls_UINT i, ls_UINT j) const
{ static char *q1="ls_array2::get_diagonal";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT ll, vdim=v.dimension();
  if((i+n<=m)||(m-i>n-j)) ll=n-j; else ll=m-i;
  if(vdim<ll) ll=vdim;
  T *aa=a+i*n+j, *vv=v.asArray();
  while(ll--) *vv=*aa, aa+=n+1, vv++;
  return *this;
}

template<class T>
const ls_array2<T>& ls_array2<T>::\
get_array(ls_array2<T>&v, ls_UINT i, ls_UINT j) const
{ static char *q1="ls_array2::get_array";
  if(!(i<m)||!(j<n)) throw ls_error(name,q1,index_error_);
  ls_UINT mm=m-i, nn=n-j, k,l,
       mv=v.number_of_rows(), nv=v.number_of_columns();
  if(!mv||!nv) throw ls_error(v.name,q1,object_is_empty_);
  if(mm>mv) mm=mv; if(nn>nv) nn=nv;
  for(k=0; k<mm; k++)
    for(l=0; l<nn; l++) v[k][l]=*this[i+k][j+l];
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::append_row(ls_UINT l)
{ static char *q1="ls_array2::append_row";
  if(!dim) throw ls_error(name,q1,object_is_empty_);
  T *aa=new T[dim+n*l]; memcpy(aa,a,sizeof(T)*dim);
  memset(aa+dim,0,sizeof(T)*n*l);
  dim+=n*l; delete[]a, a=aa, m+=l;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::append_column(ls_UINT l)
{ static char *q1="ls_array2::append_column";
  if(!dim) throw ls_error(name,q1,object_is_empty_);
  T *aa=new T[dim+=m*l], *b=a, *bb=aa;
  memset(aa,0,sizeof(T)*dim);
  ls_UINT ll=(ls_UINT)sizeof(T)*n, kk=m;
  while(kk--) memcpy(bb,b,ll),bb+=(n+l), b+=n;
  delete[]a; a=aa, n+=l;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::append_array(ls_UINT mm, ls_UINT nn)
{ ls_array2<T> v(m+mm,n+nn); v.put_array(*this);
  remove(); swap(v); return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::remove_row(ls_UINT i)
{ static char *q1="ls_array2::remove_row";
  if(!(i<m)) throw ls_error(name,q1,index_error_);
  T *aa=new T[dim-n];
  if(i) memcpy(aa,a,sizeof(T)*i*n);
  if(m-i-1) memcpy(aa+i*n,a+(i+1)*n,sizeof(T)*(m-i-1)*n);
  delete []a; a=aa, dim-=n, m--;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::remove_column(ls_UINT j)
{ static char *q1="ls_array2::remove_column";
  if(!(j<n)) throw ls_error(name,q1,index_error_);
  T *aa=new T[dim-m], *an=aa, *al=a;
  ls_UINT l=m, nn=n-1, ja=(ls_UINT)sizeof(T)*j, je=(ls_UINT)sizeof(T)*(n-j-1);
  if(!ja)
    while(l--) memcpy(an,al+1,je), an+=nn, al+=n;
  else
    if(!je)
      while(l--) memcpy(an,al,je), an+=nn, al+=n;
    else
      while(l--)
        memcpy(an,al,ja), memcpy(an+j,al+j+1,je), an+=nn, al+=n;
  delete []a; a=aa, dim-=m, n--;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::remove()
{ if(dim) dim=m=n=0, delete[]a;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::swap_row(ls_UINT i, ls_UINT k)
{ static char *q1="ls_array2::swap_row";
  if(!(i<m)||!(k<m)) throw ls_error(name,q1,index_error_);
  if(i==k) return *this;
  T s, *ai=a+i*n, *ak=a+k*n, *ae=ai+n;
  while(ai<ae) s=*ai, *ai=*ak, *ak=s, ai++, ak++;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::swap_column(ls_UINT i, ls_UINT k)
{ static char *q1="ls_array2::swap_column";
  if(!(i<n)||!(k<n)) throw ls_error(name,q1,index_error_);
  if(i==k) return *this;
  T s, *ai=a+i, *ak=a+k, *ae=ai+dim;
  while(ai<ae) s=*ai, *ai=*ak, *ak=s, ai+=n, ak+=n;
  return *this;
}

template<class T>
const ls_array2<T>& ls_array2<T>::write_row(ostream &f) const
{ static char *q1="ls_array2::write_row";
  if(!dim) throw ls_error(q1,name,object_is_empty_);
  T *aa, *ae, *ac;
  f <<name<<endl<<"{ "<<number_of_rows_<<" "<<m<<" "\
    <<number_of_columns_<<" "<<n<<endl;
  for (aa=a, ae=a+dim; aa<ae; f<<endl)
    for (ac=aa+n; aa<ac; aa++)
      if(*aa<0) f << " " << *aa; else f << "  " << *aa;
  f<<"}"<<endl;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::read_row(istream &f)
{ static char *q1="ls_array2::read_row";
  if(dim) throw ls_error(name, q1, object_not_empty_);
  char w[ls_len];
  if(*name) while(f>>w&&strcmp(w,name)); else f>>name;
  if(!f.good()) throw ls_error(name, q1, object_not_found_);
  f>>w;
  if(strcmp(w, "{")) throw ls_error(name,q1,object_contents_error_);
  if(!(f>>w)||strcmp(w, number_of_rows_)||!(f>>m))
    throw ls_error(name, q1, object_contents_error_);
  if(!(f>>w)||strcmp(w, number_of_columns_)||!(f>>n))
    throw ls_error(name, q1, object_contents_error_);
  a=new T[dim=m*n];
  for (T *aa=a, *ae=a+dim; aa<ae; f>>*aa, aa++);
  if(!(f>>w)||(*w!='}')) throw ls_error(name, q1, object_end_error_);
  return *this;
}

template<class T>
const ls_array2<T>& ls_array2<T>::write_column(ostream &f) const
{ static char *q1="ls_array2::write_column";
  if(!dim) throw ls_error(q1,name,object_is_empty_);
  T *aa, *ae, *ac, *ar;
  f <<name<<endl<<"{ "<<number_of_columns_<<" "<<n<<" "\
    <<number_of_rows_<<" "<<m<<endl;
  for (aa=a, ac=a+n, ae=a+dim; aa<ac; f<<endl, aa++)
    for (ar=aa; ar<ae; ar+=n);
    if(*ar<0) f << " " << *ar; else f << "  " << *ar;
  f<<"}"<<endl;
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::read_column(istream &f)
{ static char *q1="ls_array2::read_column";
  if(dim) throw ls_error(name, q1, object_not_empty_);
  char w[ls_len];
  T *aa, *ae, *ac, *ar;
  if(*name) while(f>>w&&strcmp(w,name)); else f>>name;
  if(!f.good()) throw ls_error(name, q1, object_not_found_);
  f>>w;
  if(strcmp(w, "{")) throw ls_error(name,q1,object_contents_error_);
  if(!(f>>w)||strcmp(w, number_of_columns_)||!(f>>n))
     throw ls_error(name, q1, object_contents_error_);
  if(!(f>>w)||strcmp(w, number_of_rows_)||!(f>>m))
    throw ls_error(name, q1, object_contents_error_);
  a=new T[dim=m*n];
  for (aa=a, ae=a+dim, ac=a+n; aa<ac; aa++)
    for (ar=aa; ar<ae; f>>*ar, ar +=n);
  if(!(f>>w)||(*w!='}')) throw ls_error(name, q1, object_end_error_);
  return *this;
}

template<class T>
const ls_array2<T>& ls_array2<T>::operator>> (char *nam) const
{ if(!*nam)
  { cout.setf(ios::fixed, ios::floatfield);
    cout.precision(int(sizeof(ls_REAL)*1.5));
    write_row(cout);
  }
else
  { ofstream f(nam);
    f.setf(ios::scientific, ios::floatfield),
    f.precision(int(sizeof(ls_REAL)*1.5)), write_row(f);
  }
  return *this;
}

template<class T>
ls_array2<T>& ls_array2<T>::operator<< (char *nam)
{ if(!*nam) read_row(cin); else{ ifstream f(nam); read_row(f);}
  return *this;
}
#endif
